
      PROGRAM MultiWell
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c
c MultiWell: a code for chemical kinetics calculations.
c Copyright (C) 2011 John R. Barker
c
c John R. Barker
c jrbarker@umich.edu
c University of Michigan
c Ann Arbor, MI 48109-2143
c (734) 763 6239
c
c This program is free software; you can redistribute it and/or
c modify it under the terms of the GNU General Public License (version 2)
c as published by the Free Software Foundation.
c
c This program is distributed in the hope that it will be useful,
c but WITHOUT ANY WARRANTY; without even the implied warranty of
c MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
c GNU General Public License for more details.
c
c See the 'ReadMe' file for a copy of the GNU General Public License,
c or contact:
c
c Free Software Foundation, Inc.
c 59 Temple Place - Suite 330
c Boston, MA 02111-1307, USA.
c
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c
c
c     VERSION HISTORY: see HISTORY file
c
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

      IMPLICIT NONE
 
      INCLUDE 'declare1.inc'
      INCLUDE 'declare2.inc'
 
      CHARACTER(len=100) TITLE
      CHARACTER(len=10)  TS
      CHARACTER(len=3)   Punits , bar , atm , tor , mcc
      CHARACTER(len=4)   POUT , Eunits , kcal , kjou , cmm1 , 
     &                   Rotatunits , DUM 
      CHARACTER (len=10)  dummy

      CHARACTER(len=20)  NameFile, NameOUT, NameSMM, NameARY, NameDIST, 
     &                    NameFLX, NameRAT

      CHARACTER(len=7) AVERSION 
      CHARACTER(len=8) ADATE 
      PARAMETER ( AVERSION='2021', ADATE='Jan 2021' )
 
      PARAMETER (bar='BAR',atm='ATM',tor='TOR', mcc='MCC' ,
     &           cmm1='CM-1',kcal='KCAL',kjou='KJOU')
 
      INTEGER i , j , k , l , Nforward , M , Ntrials , Igo , IPP , II , 
     &        Mol , Nreac , ito , Jr , Icalc , IR , Molinit , JJ , ki ,
     &        Mdum , Np , Ip , IDUM0 , ncen , lenstr , KK ,
     &        kkey , ntunflag , nivrflag , Pressflag , NSTOP , iflag     
      INTEGER kkmax
      INTEGER NCUM
      INTEGER ndoubt
 
      DOUBLE PRECISION Temp , P(50) , PP(50) , Sig , Eps , Amu , Sigma ,
     &                 Epsil , Mass , pi , Time , E , XRANST , AA , EE ,
     &                 Einit , RR , SigM , EpsM , AmuM , CollFreq , EM
     
      INTEGER jbim, bt
      DOUBLE PRECISION Af, Bx, xd
 
      DOUBLE PRECISION FracMol(MaxWell+MaxProd) , AveE(MaxWell+MaxProd)
     &                 , Test , SCALE , numbdens , Tvib , NumTot ,
     &                 klj , kqm ,Tread , kcol , DD , DR , DOUBNTERP ,
     &                 Rtrials , Colli , xref
 
      DOUBLE PRECISION RAN1 , Etherm , PDOWN , Estart , 
     &                 Kinf(MaxWell,MaxChan) , Ainf(MaxWell,MaxChan) , 
     &                 Einf(MaxWell,MaxChan) , Var(MaxWell+MaxProd) , 
     &                 Pfact , Rob_1 , Delt , 
     &                 Flux(MaxWell,MaxChan,Ntime),
     &                 kosc(MaxWell,MaxChan) , qele

      CHARACTER(len=8) KEYWORD , KEYTEMP
      DIMENSION        KEYWORD(5) , DUM(3)
 
      DOUBLE PRECISION  vivr , vave , civr , pivr , tivr
      DOUBLE PRECISION ddum
      INTEGER     jivr , jtun ,  iivr
      INTEGER     III, Iwell , ichan
      INTEGER     iwellindex, Nreaindex(MaxWell*MaxChan)
      INTEGER     Nproindex(MaxWell*MaxChan)
      COMMON/IVR/ vivr( MaxWell , MaxChan ) , 
     &            vave( MaxWell , MaxChan ) ,
     &            pivr( MaxWell , MaxChan ) ,
     &            tivr( MaxWell , MaxChan ) ,
     &            civr( MaxWell , MaxChan , 3 ) , 
     &            iivr( MaxWell , MaxChan )

      EXTERNAL RAN1 , Etherm , PDOWN , Estart

      SAVE 

      pi = 4.0D+00*DATAN(1.0D+00)
c
c     FOR USER-DEFINED FILENAME ("NameFile")
c
      CALL get_command_argument( 1, NameFile )
      NameFile=NameFile(1:len_trim(NameFile))
      IF(NameFile.eq.'') NameFile="multiwell.dat"         ! default filename
      NameOUT=NameFile(1:len_trim(NameFile)-4)//".out"
      NameSMM=NameFile(1:len_trim(NameFile)-4)//".sum"
      NameARY=NameFile(1:len_trim(NameFile)-4)//".array"
      NameDIST=NameFile(1:len_trim(NameFile)-4)//".dist"
      NameRAT=NameFile(1:len_trim(NameFile)-4)//".rate"
      NameFLX=NameFile(1:len_trim(NameFile)-4)//".flux"
c
c     I/O Unit numbers
c
      KIN   = 2  ! Data file
      KSTD  = 6  ! standard output (screen)
      KSMM  = 7  ! Short summary of output file 
      KARY  = 8  ! Summary of array variables
      KOUT  = 9  ! General time-dependent output
      KRAT  = 10 ! average rate 'constants'
      KFLX  = 11 ! relative reactive flux via a given path
      KDIS  = 12 ! vib distribution

      OPEN (KIN,FILE=NameFile,STATUS='OLD')          ! Data file
      OPEN (KOUT,FILE=NameOUT,STATUS='UNKNOWN')      ! General time-dependent output
      OPEN (KSMM,FILE=NameSMM,STATUS='UNKNOWN')      ! Short summary output for fall-off calculations
      OPEN (KARY,FILE=NameARY,STATUS='UNKNOWN')      ! Summary of input, output, and intermediate array variables
      OPEN (KRAT,FILE=NameRAT,STATUS='UNKNOWN')      ! average rate 'constants'
      OPEN (KFLX,FILE=NameFLX,STATUS='UNKNOWN')      ! relative reactive flux via a given reaction path
  
      READ (KIN,99003) TITLE

      READ (KIN,*) Egrain1 , imax1 , Isize , Emax2 , IDUM

      IF ( Isize .GT. Imax ) THEN
         WRITE (KSTD,99002) Isize
         WRITE (KOUT,99002) Isize
         STOP
      ENDIF
      IF ( imax1 .GT. Isize-2 ) THEN
       WRITE (KSTD,*) '*** FATAL:  Imax1 must be  (Isize-2) ***'
       WRITE (KOUT,*) '*** FATAL:  Imax1 must be  (Isize-2) ***'
       STOP
      ENDIF      
      
      DO ki = 7 , 11
        WRITE (ki,99001) AVERSION , ADATE , AVERSION , ADATE
        CALL DateTime(ki)
        WRITE (ki,99003) TITLE
      END DO

      Emax1 = Egrain1*(imax1-1)
      Egrain2 = Emax2/(Isize-imax1-1)
c.......................................................................
c     Energy grain parameters for interpolation 'double arrays'
c
c     'Double arrays' have two sections:
c          segment 1 consists of equally spaced (Egrain1) values ranging
c               from E=0 to Emax1
c          segment 2 consists of equally spaced values from E=0 to Emax2;
c               the spacing depends on the number of array elements
c               remaining: Isize-Emax1/Egrain1-1
c
c     Egrain1     = energy grain of lower segments in 'double array' (units: cm-1)
c     Emax1     = maximum energy of 1st segment of double array (units: cm-1)
c     Emax2     = maximum energy of 2nd segment of double array (units: cm-1)
c     IDUM     = random number seed (positive, non-zero integer)
c.......................................................................

      DistStep = Emax2/(Ndist-1)                ! Energy step for binned distributions; used in book1 and output
      DistStep = INT( DistStep/Egrain1 + 1 )*Egrain1

      IF ( IDUM.EQ.0 ) IDUM = 1990339   ! Random number seed
      IDUM0 = IDUM
      IDUM = -ABS(IDUM)
      XRANST = RAN1(IDUM)    ! initialize RAN1

      DO ki = 7 , 11
        WRITE (ki,99044) Egrain1 , Emax1 , imax1 , Egrain2 , Emax2 , 
     &                   (Isize-imax1)
      END DO
      
c.......................................................................
c     Key words for units
c    READ Punits , Eunits , Rotatunits
c
c     Punits       = pressure units key word [use upper case CHARACTER*3]
c                 = 'BAR', 'ATM', 'TOR' [also will accept 'TORR'], or 'MCC'
c     Eunits       = energy units keyword [use upper case CHARACTER*4]
c                 = 'CM-1','KCAL', or 'KJOU' (for cm-1, kcal/mole, or kJ/mole)
c     Rotatunits  = units for moment of inertia 
c                = 'AMUA', 'GMCM', 'CM-1', MHz, 'GHz'
c.......................................................................
      READ (KIN,*) DUM(1) , DUM(2) , DUM(3)
      CALL ucase ( DUM(1) )   ! convert to upper case
      CALL ucase ( DUM(2) )   ! convert to upper case
      CALL ucase ( DUM(3) )   ! convert to upper case
      CALL UnitTest ( DUM , Punits , Eunits , Rotatunits , KOUT )
c.......................................................................
c     Physical parameters
c
c     Temp     = translational temperature (units: degrees Kelvin)
c     Tvib     = vibrational temperature (units: degrees Kelvin)
c     Np     = number of pressures
c     PP     = vector of Np pressures
c.......................................................................
      READ (KIN,*) Temp , Tvib
      READ (KIN,*) Np              ! number of pressures (or number densities)
      READ (KIN,*) (PP(i),i=1,Np)
      IF ( Np.GT.50 ) THEN
         write(*,*) 'FATAL: Only 50 pressures allowed....'
         write(KOUT,*) 'FATAL: Only 50 pressures allowed....'
         STOP
      ENDIF
c.......................................................................
c
c     Convert pressures to bars
c
c.......................................................................
      IF ( Punits.EQ.atm ) THEN
         POUT = 'atm '
         Pfact = 1.01325
      ELSEIF ( Punits.EQ.tor ) THEN
         POUT = 'Torr'
         Pfact = 1.01325/760.
      ELSEIF ( Punits.EQ.bar ) THEN
         POUT = 'bar '
         Pfact = 1.0
      ELSEIF ( Punits.EQ.mcc ) THEN
         POUT = '1/cc'
         Pfact = Temp/7.2428D+21
      ELSE
         WRITE(*,*) 'FATAL: Pressure units not specified properly'
         WRITE(KOUT,*) 'FATAL: Pressure units not specified properly'
         STOP
      ENDIF
 
      DO 100 i = 1 , Np
         P(i) = PP(i)*Pfact            ! P(i) converted to bar
 100  CONTINUE
 
      READ (KIN,*) NWells , NProds
      DO 200 i = 1 , NWells
         READ (KIN,*) IMol(i) , MolName(i) , HMol(i) , MolMom(i) , 
     &                Molsym(i) , Molele(i) , Molopt(i)
c         MolName(i) = ADJUSTR( MolName(i) )

         IF ( Eunits.EQ.kcal ) THEN
            HMol(i) = HMol(i)*caltocm           ! Convert kcal/mole to cm-1
         ELSEIF ( Eunits.EQ.kjou ) THEN
            HMol(i) = HMol(i)*jtocm             ! Convert kJ/mole to cm-1
         ENDIF
         Nchan(i) = 0                           ! Initialize Nchan
         call rotunits( Rotatunits , MolMom(i) , 'AMUA' )     ! convert rotational units to 'AMUA'
200   CONTINUE
 
      DO 300 i = NWells + 1 , NWells + NProds
         READ (KIN,*) IMol(i) , MolName(i) , Hmol(i)
c         MolName(i) = ADJUSTR( MolName(i) )
         IF ( Eunits.EQ.kcal ) THEN
            HMol(i) = HMol(i)*caltocm           ! Convert kcal/mole to cm-1
         ELSEIF ( Eunits.EQ.kjou ) THEN
            HMol(i) = HMol(i)*jtocm           ! Convert kJ/mole to cm-1
         ENDIF
 300  CONTINUE
c.......................................................................
c     Reactant/Product parameters
c
c     NWells     = number of 'wells'
c     NProds     = number of entrance/exit species
c
c     IMol     = index number for molecule
c     MolName     = name (10 characters) of molecule
c     HMol     = enthalpy of formation at 0K
c     MolMom     = moment of inertia for 2-dim external rotor (amu Ang^2), 
c                gram*cm^2, cm-1, MHz, or GHz
c     Molsym     = external symmetry number for molecule
c     Molele     = electronic partition function for molecule
c     Molopt     = number of optical isomers for molecule
c     Iread     =1: read densities from file
c            *** filename: same name as MolName +'.dens'
c            ***    e.g. 'Benzene.dens'
c.......................................................................
      CALL DensArray                            ! Calculate or read densities
 
      DO ki = 7 , 9
        WRITE (ki,99031) Temp , Tvib , NWells , NProds
        WRITE (ki,99036) (i,i=1,NWells+NProds)
        WRITE (ki,99035) (ADJUSTR(MolName(i)),i=1,NWells+NProds)
        WRITE (ki,99023) (HMol(i),i=1,NWells+NProds)
        WRITE (ki,99025) (HMol(i)/jtocm,i=1,NWells+NProds)
        WRITE (ki,99024) (HMol(i)/caltocm,i=1,NWells+NProds)
        WRITE (ki,99033) (Viblo(i),i=1,NWells)
        WRITE (ki,99045) (MolMom(i),i=1,NWells)
        WRITE (ki,99046) (Molsym(i),i=1,NWells)
        WRITE (ki,99047) (Molele(i),i=1,NWells)
        WRITE (ki,99048) (Molopt(i),i=1,NWells)
        WRITE (ki,99041) (Etherm(i,Temp),i=1,NWells)
        WRITE (ki,99009)
      END DO
 
      WRITE (KARY,99010) (ADJUSTR(MolName(i)),i=1,NWells)

      DO 400 II = 1 , Isize
         IF ( II.LE.imax1 ) THEN
            E = (II-1)*Egrain1
         ELSE
            E = (II-imax1-1)*Egrain2
         ENDIF
         WRITE (KARY,99011) E , (exp(Dens(i,II)),i=1,NWells)
                                                        ! Densities stored as logs
 400  CONTINUE
 
c.......................................................................
c     Collision parameters - collider
c
c     SigM     L-J sigma (Ang) for collider
c     EpsM     L-J Eps (Kelvins) for collider
c     AmuM     Molecular weight (g/mole) of collider
c     ETKEY    Keyword to designate energy transfer treatment:
c              = OLDET     traditional treatment of energy transfer at low E
c              = NEWET     Barker's 'New Approach" to energy transfer
c                [J.R. Barker, Int. J. Chem. Kinetics, 41, 748-763 (2009)]
c.......................................................................

      DO ki = 7 , 9
        WRITE(ki,99073) NWells
      END DO

      READ (KIN,*) SigM , EpsM , AmuM , Amu
      READ (KIN, 99173)  ETKEY
c      WRITE (*,*) ETKEY
      call ucase ( ETKEY)
        
      DO ki = 7 , 9
        IF     ( ETKEY.EQ.newet ) THEN
          WRITE(ki,*) 'BARKER''S ''NEW APPROACH'' TO ENERGY TRANSFER'
          WRITE(ki,*) 
     &       '[J.R. Barker, Int. J. Chem. Kinetics, 41, 748-763 (2009)]'
        ELSEIF ( ETKEY.EQ.oldet ) THEN
          WRITE(ki,*) 'TRADITIONAL APPROACH TO ENERGY TRANSFER'
        ELSEIF ( ETKEY.NE.newet .AND. ETKEY.NE.oldet ) THEN
          ETKEY=xxxet
          WRITE(ki,*) 'UNSPECIFIED ET TREATMENT: USE BARKER''S APPROACH'
          WRITE(ki,*) 
     &       '[J.R. Barker, Int. J. Chem. Kinetics, 41, 748-763 (2009)]'
        ENDIF
        WRITE (ki,99055)
        Mdum = 0
        WRITE (ki,99056) Mdum , SigM , EpsM , AmuM
      END DO
      IF (ETKEY.EQ. xxxet ) THEN      ! not specified by user
        BACKSPACE (KIN)
        ETKEY = newet 
      ENDIF
c      WRITE (*,*) ETKEY
       
      DO 500 j = 1 , NWells
         READ (KIN,*) Mol , Sig , Eps , ITYPE(Mol) , (DC(Mol,i),i=1,8)
         READ (KIN,*) LJQM(Mol)
         call ucase( LJQM(Mol) )
c.......................................................................
c     Collision parameters
c
c     Sig     L-J sigma (Ang) for molecule
c     Eps     L-J Eps (Kelvins) for molecule
c     Amu     Molecular weight (g/mole) of molecule
c     ITYPE     selects model type in Subroutine PDOWN
c     DC     coefficients for collision model
c     LJQM     = 'LJ' for Lennard-Jones collision frequency, ALL collisions inelastic
c          = 'QM' for total cross section collision frequency
c.......................................................................
c
         Sigma = 0.5*(SigM+Sig)
         Epsil = SQRT(EpsM*Eps)
         Mass = AmuM*Amu/(AmuM+Amu)
 
c         Colli = 1.0d+00/(0.636+0.246*LOG(Temp/Epsil))                ! simple collision integral [Troe, J. Chem. Phys. 66, 4758 (1977)]

         Colli = 1.16145/((Temp/Epsil)**0.14874) +                    ! better collision integral [Reid, Prausnitz, and Sherwood, Their Properties
     &              0.52487*exp(-0.7732*Temp/Epsil) +                 !   of Gases and Liquids: Their Estimaation and Correlation, 3rd Edition,
     &              2.16178*exp(-2.437887*Temp/Epsil)                 !   McGraw-Hill, New York, 1977]

         klj = pi*1.E-16*(Sigma**2)*1.45525E+04*SQRT(Temp/Mass)*Colli
 
         kqm = 7.64E-12*((Epsil*Sigma**6)**0.4)*(Temp/Mass)**0.3

         QCollK( Mol ) = kqm                                          ! Max quantum rate constant

         IF ( LJQM(Mol).EQ.LJ ) THEN
            CollK(Mol) = klj
         ELSE
            CollK(Mol) = kqm
         ENDIF
         
c.......................................................................
c
c     klj = L-J collision rate constant (s-1)
c     kqm = collision rate constant based on the total quantum cross section
c          [Durant and Kaufman, Chem. Phys. Letters, 142, 246 (1987)]
c.......................................................................
c
      DO ki = 7 , 9
         WRITE (ki,99056) Mol , Sig , Eps , Amu , Sigma , Epsil , 
     &                      Mass , klj , kqm , CollK(Mol)
      END DO
      
 500  CONTINUE
c
c     Call PDOWN WITH E=-1.0 in order to store LI(Mol,i) in COMMON for print-out
c
      WRITE (KOUT,99030)
      WRITE (KSMM,99030)
      WRITE (KARY,99030)
 
      DO Mol = 1 , NWells
         CALL COLNORM(Mol,Temp)         ! Calculate COLLUP and CNORM arrays
      END DO

c.......................................................................
c     Reaction parameters
c
c     Nforward= number of reaction parameters to be input (forward rxns, only)
c     Mol     = index of reactant molecule
c     ito     = Jto      = index of product molecule
c     TS     = TSname = Name of transition state (up to 10 characters)
c     RR     = TSmom  = moment of inertia (amu.ang^2, gram.cm^2, cm-1, MHz, GHz)
c     j     = TSsym  = external symmetry number for TS
c     qele     = TSele      = electronic partition function for TS
c     l     = TSopt      = number of optical isomers for TS
c     AA     = Afact     = A-factor for reaction (units: s-1)
c     EE     = Eo     = critical energy, relative to ZPE of reactant
c
c       KEYWORDS (5 are expected):
c
c         'NOCENT'   no centrifugal correction             [NCENT=  0]
c         'CENT1'    1-D adiabatic rotor centrifugal       [NCENT=  1]
c         'CENT2'    2-D adiabatic rotor centrifugal       [NCENT=  2]
c         'CENT3'    2-D adiabatic rotor centrifugal       [NCENT=  3]
c         'CENTX'    LEGACY centrifugal correction         [NCENT= -2] (not recommended)
c
c         'NOREV'    neglecting the reverse reaction        [Jrev = Jr]
c         'REV'      calculating reverse rxn rate; after calculation,
c                    Jrev=1 is changed to Jrev=2
c
c         'FAST'     neglecting limitations due to IVR
c         'SLOW'     including IVR limitations; a following line contains 
c                        'SLOW', pivr(Mol,i), vivr(Mol,i), vave(Mol,i), 
c                             tivr(Mol,i), civr(Mol,i,1), civr(Mol,i,2), 
c                             civr(Mol,i,3)
c
c         'NOTUN'    for neglecting tunneling
c         'TUN'      for including tunneling via unsymmetrical Eckart barrier;
c                       a following line contains: 'TUN', tunnel(Mol,i)
c
c          'ILT'     Inverse laplace transform method for k(E)
c          'SUM'     File containing sums of states (usually generated by Densum)
c          'CRP'     File containing CRP (usually generated by SCTST)
c          'RKE'     File containing k(E), generated externally
c          'REVRKE'  k(E) for reverse reaction calculated from k(E) file for forward rxn
c
c                Jread = 0 for ILT method
c                      = 1 for reading sums of states from external file
c                        *** filename: same name as TSame +'.dens'
c                        ***   e.g. 'TS-1.dens'
c                      = 2 for reading k(E) from external file
c                        *** filename: same name as TSame +'.rke'
c                        ***   e.g. 'TS-1.rke'
c                      = -2 for calculating reverse k(E) from external file for forward k(E)
c                        *** filename: same name as TSame +'.rke'
c                        ***   e.g. 'TS-1.rke'
c                      = 3 for reading CRP of states from external file CREATED BY SCTST or PARSCTST
c                        *** filename: same name as TSame +'.dens'
c                        ***   e.g. 'TS-1.dens'
c.......................................................................
c
c
c     Nchan(Mol) = number of rxn channels originating from a given molecule
c.......................................................................
  
      READ (KIN,*) Nforward
      Nreac = Nforward
 
      IF ( Nforward .GT. 0 ) WRITE(KOUT,99074) Nforward

      ntunflag = 0                     ! flag for tunneling
      nivrflag = 0                     ! flag for slow IVR
      Pressflag = 0                    ! flag for reactant [B]  (for microcanonical pseudo-first order rxn)

      DO 800 i = 1 , Nforward
 
         READ (KIN,*) Mol , ito , TS , RR , j , qele ,
     &                 l , AA , EE , (KEYWORD(II),II=1,5)
         DO II = 1 , 5
            call ucase( KEYWORD(II) )
         END DO
         WRITE (KOUT,99075) i , Mol , ito , TS , RR , j , qele , 
     &                            l , AA , EE , (KEYWORD(II),II=1,5)
     
         call rotunits( Rotatunits , RR , 'AMUA' )     ! convert rotational units to 'AMUA'

         Nreaindex(i)=Mol ! keep track of reactants and products so that
         Nproindex(i)=ito ! the initial distribution and Eor can be printed
                          ! correctly in multiwell.array

         Nchan(Mol) = Nchan(Mol) + 1
         Jto(Mol,Nchan(Mol)) = ito
         TSname(Mol,Nchan(Mol)) = TS
         TSmom(Mol,Nchan(Mol)) = RR
         TSsym(Mol,Nchan(Mol)) = j
         TSele(Mol,Nchan(Mol)) = qele
         TSopt(Mol,Nchan(Mol)) = l
         Afac(Mol,Nchan(Mol)) = AA
         IF(Afac(Mol,Nchan(Mol)).lt.0) THEN
           Press(Mol,Nchan(Mol))=ABS(Afac(Mol,Nchan(Mol)))
           Pressflag=Pressflag+1                            ! (for microcanonical pseudo-first order rxn)
          ELSE
           Press(Mol,Nchan(Mol))=1
         ENDIF
         IF ( Eunits.EQ.cmm1 ) THEN
            Eo(Mol,Nchan(Mol)) = EE                     ! cm-1 units
         ELSEIF ( Eunits.EQ.kcal ) THEN
            Eo(Mol,Nchan(Mol)) = EE*caltocm             ! Convert kcal/mole to cm-1
         ELSEIF ( Eunits.EQ.kjou ) THEN
            Eo(Mol,Nchan(Mol)) = EE*jtocm               ! Convert kJ/mole to cm-1
         ENDIF
         Hts(Mol,ito) = HMol(Mol) + Eo(Mol,Nchan(Mol))  ! Rxn barrier (or transition state) enthalpy same in both directions
c
c      KEYWORDS
c
c
c        DEFAULTS
c
         NCENT(Mol,Nchan(Mol)) = 2		! CENT2 centrifugal correction
         Jr = 1							! reversible reaction
         jtun = 0						! no tunneling
         jivr = 0						! fast ivr
         itun(Mol,Nchan(Mol)) = 0
         Jread(Mol,Nchan(Mol)) = 1		! read from sum of states file (suffix .dens)

         kkey = 0

         DO 701 KK = 1 , 5
           call ucase( KEYWORD(KK) )

           IF     (KEYWORD(KK) .EQ. 'ILT') THEN   ! k(E) from Inverse Laplace Transform method
              Jread(Mol,Nchan(Mol)) = 0
              kkey = kkey + 1
           ELSEIF (KEYWORD(KK) .EQ. 'SUM') THEN   ! external sums of states file
              Jread(Mol,Nchan(Mol)) = 1
              kkey = kkey + 1
           ELSEIF (KEYWORD(KK) .EQ. 'RKE') THEN  ! external k(E) file
              Jread(Mol,Nchan(Mol)) = 2
              kkey = kkey + 1
           ELSEIF (KEYWORD(KK) .EQ. 'CRP') THEN   ! external CRP file (like a sums file) generated by SCTST
              Jread(Mol,Nchan(Mol)) = 3
              kkey = kkey + 1
           ELSEIF (KEYWORD(KK) .EQ. 'REVRKE') THEN  ! external k(E) file used to calculate REVERSE reaction
              Jread(Mol,Nchan(Mol)) = -2
              kkey = kkey + 1
           ELSEIF (KEYWORD(KK) .EQ. 'REV') THEN   ! Include reverse reaction
              Jr = 1
              kkey = kkey + 1
           ELSEIF (KEYWORD(KK) .EQ. 'NOREV') THEN
              Jr = 0
              kkey = kkey + 1
           ELSEIF (KEYWORD(KK) .EQ. 'TUN') THEN   ! Include tunneling
              ntunflag = 1
              jtun = 1
              kkey = kkey + 1
           ELSEIF (KEYWORD(KK) .EQ. 'NOTUN') THEN  ! No tunneling
              jtun = 0  
              kkey = kkey + 1
           ELSEIF (KEYWORD(KK) .EQ. 'SLOW') THEN   ! Include slow IVR
              nivrflag = 1
              jivr = 1
              kkey = kkey + 1
           ELSEIF (KEYWORD(KK) .EQ. 'FAST') THEN   ! Fast IVR
              jivr = 0
              kkey = kkey + 1
           ELSEIF (KEYWORD(KK) .EQ. 'NOCENT') THEN   ! No centrifugal correction
              NCENT(Mol,Nchan(Mol)) = 0
              kkey = kkey + 1
           ELSEIF (KEYWORD(KK) .EQ. 'CENTX') THEN   ! Include LEGACY centrifugal correction
              NCENT(Mol,Nchan(Mol)) = -2
              kkey = kkey + 1
           ELSEIF (KEYWORD(KK) .EQ. 'CENT1') THEN   ! Include 1-D ROTOR centrifugal correction
              NCENT(Mol,Nchan(Mol)) = 1
              kkey = kkey + 1
           ELSEIF (KEYWORD(KK) .EQ. 'CENT2') THEN   ! Include 2-D ROTOR centrifugal correction
              NCENT(Mol,Nchan(Mol)) = 2
              kkey = kkey + 1
           ELSEIF (KEYWORD(KK) .EQ. 'CENT3') THEN   ! Include 3-D ROTOR centrifugal correction
              NCENT(Mol,Nchan(Mol)) = 3
              kkey = kkey + 1
           ENDIF

 701     CONTINUE

           NSTOP = 0
           IF ( kkey .NE. 5 ) THEN
             write(6,*)'*** LESS THAN 5 REQUIRED KEYWORDS ***'
             write(KOUT,*)'*** LESS THAN 5 REQUIRED KEYWORDS ***'
             NSTOP = 1
           ENDIF
           IF ((Jread(Mol,Nchan(Mol)).eq.0).AND.(jtun.EQ.1))THEN
             write(6,*)'KEYWORDS: ILT CANNOT BE USED WITH TUN'
             write(KOUT,*)'KEYWORDS: ILT CANNOT BE USED WITH TUN'
             NSTOP = 1
           ENDIF
           IF ((Jread(Mol,Nchan(Mol)).eq.0).AND.
     &           (NCENT(Mol,Nchan(Mol)).NE.0))THEN
             write(6,*)'KEYWORDS: ILT CANNOT BE USED WITH CENT'
             write(KOUT,*)'KEYWORDS: ILT CANNOT BE USED WITH CENT'
             NSTOP = 1
           ENDIF
           IF ( ( NCENT(Mol,Nchan(Mol)) .NE. 0 ) .AND.
     &           (TSmom(Mol,Nchan(Mol)) .LE. 0.0))THEN
             write(6,*)'CENT KEYWORD REQUIRES TSmom >0'
             write(KOUT,*)'CENT KEYWORD REQUIRES TSmom >0'
             NSTOP = 1
           ENDIF
           IF((Jread(Mol,Nchan(Mol)).eq.2).AND.(jtun.EQ.1))THEN
             write(6,*)'KEYWORDS: RKE CANNOT BE USED WITH TUN'
             write(KOUT,*)'KEYWORDS: RKE CANNOT BE USED WITH TUN'
             NSTOP = 1
           ENDIF
           IF((Jread(Mol,Nchan(Mol)).eq.3).AND.(jtun.EQ.1))THEN
             write(6,*)'KEYWORDS: TUN CANNOT BE USED WITH CRP'
             write(6,*)'   BECAUSE CRP ALREADY INCLUDES TUNNELING'
             write(KOUT,*)'KEYWORDS: TUN CANNOT BE USED WITH CRP'
             write(KOUT,*)'   BECAUSE CRP ALREADY INCLUDES TUNNELING'
             NSTOP = 1
           ENDIF
           IF((NCENT(Mol,Nchan(Mol)).NE. 0).AND.(jtun.EQ.1))THEN
             write(6,*)'KEYWORDS: CENT CANNOT BE USED WITH TUN'
             write(KOUT,*)'KEYWORDS: CENT CANNOT BE USED WITH TUN'
             NSTOP = 1
           ENDIF
           IF((Jread(Mol,Nchan(Mol)).eq.-2).AND.(jtun.EQ.1))THEN
             write(6,*)'KEYWORDS: RKE CANNOT BE USED WITH TUN'
             write(KOUT,*)'KEYWORDS: RKE CANNOT BE USED WITH TUN'
             NSTOP = 1
           ENDIF

           IF ( NSTOP .NE. 0 ) THEN
             write(*,*)'***** FATAL ERROR(S) (NSTOP .NE. 0)'
             write(KOUT,*)'***** FATAL ERROR(S) (NSTOP .NE. 0)'
             STOP
           END IF
             
          IF (jivr .EQ. 1 ) THEN
             iivr(Mol,Nchan(Mol)) = 1
             READ(KIN,*) KEYTEMP, vivr(Mol,Nchan(Mol)), 
     &          vave(Mol,Nchan(Mol)) ,
     &          pivr( Mol , Nchan(Mol) ) , tivr( Mol , Nchan(Mol) ) ,
     &          ( civr( Mol , Nchan(Mol) , ii ) , ii = 1 , 3 )
          ENDIF

          IF (jtun .EQ. 1 ) THEN
             itun(Mol,Nchan(Mol)) = 1
             READ(KIN,*) KEYTEMP , vimag(Mol,Nchan(Mol))
          ENDIF

         IF ( Jr.EQ.1 ) THEN                    ! Jr=1 for rev rxn
            Jrev(Mol,Nchan(Mol)) = 1
            IF ( ito .GT. NWells ) THEN
               Jrev(Mol,Nchan(Mol)) = 0         ! If 'ito' is an irreversible product molecule
            ELSE
               Hts(ito,Mol) = Hts(Mol,ito)      ! Rxn barrier (or transition state) enthalpy same in both directions
               Nreac = Nreac + 1
               Nchan(ito) = Nchan(ito) + 1
               Jto(ito,Nchan(ito)) = Mol
               TSname(ito,Nchan(ito)) = TS
               TSmom(ito,Nchan(ito)) = RR
               Afac(ito,Nchan(ito)) = AA
               Eo(ito,Nchan(ito)) = Hts(ito,Mol) - HMol(ito)
               NCENT(ito,Nchan(ito)) = NCENT(Mol,Nchan(Mol))
               TSsym(ito,Nchan(ito)) = j
               TSele(ito,Nchan(ito)) = qele
               TSopt(ito,Nchan(ito)) = l
               Jrev(ito,Nchan(ito)) = 0
               iivr(ito,Nchan(ito)) = iivr(Mol,Nchan(Mol))
               vivr(ito,Nchan(ito)) = vivr(Mol,Nchan(Mol))
               vimag(ito,Nchan(ito)) = vimag(Mol,Nchan(Mol))
               iivr(ito,Nchan(ito)) = iivr(Mol,Nchan(Mol))
               pivr(ito,Nchan(ito)) = pivr(Mol,Nchan(Mol))
               tivr(ito,Nchan(ito)) = tivr(Mol,Nchan(Mol))
               vave(ito,Nchan(ito)) = vave(Mol,Nchan(Mol))
               civr(ito,Nchan(ito),1) = civr(Mol,Nchan(Mol),1)
               civr(ito,Nchan(ito),2) = civr(Mol,Nchan(Mol),2)
               civr(ito,Nchan(ito),3) = civr(Mol,Nchan(Mol),3)
               itun(ito,Nchan(ito)) = itun(Mol,Nchan(Mol))
              
               IF ( Jread(Mol,Nchan(Mol)).EQ.2 ) THEN
                  Jread(ito,Nchan(ito)) = -2             ! now, RateArray will open and read k(E) for forward rxn and then calculate reverse k(E)
               ELSE
                  Jread(ito,Nchan(ito)) = Jread(Mol,Nchan(Mol))
               ENDIF
            ENDIF
         ENDIF
           

 800  CONTINUE ! end of Nforward reactions
 
c------------------------------------------------------------------------------------------
c	SUPPLEMENTRY REACTIONS (IRREVERSIBLE)
c------------------------------------------------------------------------------------------
c	First (unimolecular) or second order (bimolecular)
c	BIMOLECULAR REACTIONS between Mol and bath gas,
c	Pseudo-first-order rate constant for reaction with bath gas is
c	the product:
c			ksup(Mol,n)*[bath gas]
c
c	where ksup(Mol,n) = Afc(Mol,n) * exp( -Bsr(Mol,n)/Temp )
c
c	MaxSup	:	maximum allowed supplementary rxns for each Well (index Mol)
c	nsux 	:	total number of supplementary reactions (all wells)
c	nsmax	:	number of supplementary rxns  for a Well (index Mol) 
c	nsup		:	index number for supplentary rxn for a Well (index Mol)
c	SupName	:	name of supplementary reaction
c	sto		:	product index
c	norder	:	reaction order
c	Afc		:	Arrhenius A-factor (s-1 or cm3 s-1)
c	Bsr		:	E(activation)/R
c	dde		:	Quench energy (energy lost to environment)
c	ksup		:	Supplemntary rate constant = Afc*exp(-Bsr/T)
c
       DO Mol = 1, NWells
         nsmax(Mol) = 0						! initialize number of supplementary reactions for each Well
       END DO
       
       READ (KIN,*) dummy
       CALL UCASE ( dummy )
       IF ( dummy .EQ. 'MORERXN' ) THEN
         READ (KIN,*) nsux                        		! total number of supplementary reactions to be read
         DO jbim = 1, nsux
           READ (KIN,*) Mol, bt , dummy, ndoubt, Af, Bx, xd		! from, to, Name, rxn_order, Afactor, Ea/R , vib energy in Well(ito) lost to bath when quenching
           IF ( Mol.GT. NWells ) THEN				! can only react with Wells
             Write(*,*) 'FATAL: Supplementary rxns only from Wells'
             STOP
           ENDIF
           nsmax(Mol) = nsmax(Mol) + 1
           nsup(Mol,nsmax(Mol)) = nsmax(Mol)
           IF ( nsmax(Mol) .GT. MaxSup ) THEN				! maximum for each Mol: MaxSup
             Write(*,*) 'FATAL: Too many supplementary rxns, Mol:', Mol
             STOP
           ENDIF
           SupName( Mol,nsmax(Mol) ) = dummy			! Rxn Name
           sto( Mol,nsmax(Mol) ) = bt                  ! index number of product (INTEGER)
           norder(Mol,nsmax(Mol) ) = ndoubt			! reaction order
           Afc( Mol,nsmax(Mol) ) = Af				! Afactor
           Bsr( Mol,nsmax(Mol) ) = Bx				! Ea/R activation energy
           dde( Mol,nsmax(Mol) ) = xd				! Quench energy (electronic lost from product to bath)
           ksup( Mol,nsmax(Mol) ) = Af*exp( -Bx/Temp )
         END DO      ! jbim
       ELSE
         BACKSPACE (KIN)
       ENDIF



c------------------------------------------------------------------------------------------
c   COMPUTE COLLISION PROBABILITIES
c------------------------------------------------------------------------------------------

      IF (Nforward .GT. 0 ) CALL RateArray(Temp)       ! Calculate microcanonical rate constants at temperature Temp

      EM = (Isize-imax1-2)*Egrain2           ! Top energy in double array
 
      DO j = 1 , NWells
         IF ( Nchan(j) .EQ. 0 ) THEN         ! New up-collision method: normalization at imax1 (if no rxn channel)
           CNORMref(j) = CNORM(j,imax1)
           Eref(j) = (imax1-1)*Egrain1
         ELSE
           CNORMref(j) = CNORM(j,imax1)
           Eref(j) = Emax1
           DD = EM                           ! Top energy in double array
           DO i = 1 , Nchan(j)
             IF ( Eor(j,i) .LT. DD .AND. Eor(j,i) .GT. Emax1 ) THEN ! New up-collision method: normalization at Eor
                DD = Eor(j,i)            ! The lowest reaction threshold that is higher than Emax1
                Eref(j) = DD
                CNORMref(j) = DOUBNTERP( DD , j , CNORM )
             ENDIF
           END DO
         ENDIF
      END DO

c------------------------------------------------------------------------------------------
c   COMPUTE RATE CONSTANTS, ETC
c------------------------------------------------------------------------------------------

      DO 900 j = 1 , NWells
         IF ( Nchan(j) .NE. 0 ) THEN
           DO 850 i = 1 , Nchan(j)            
             kcol = CollK(j)
             if(itun(j,i).eq.1)then
               CALL QKinftun(j,i,Temp,Kinf(j,i),Ainf(j,i),Einf(j,i))    ! high-pressure rate constant w/ tunneling effects
             else
               CALL QKinf(j,i,Temp,Kinf(j,i),Ainf(j,i),Einf(j,i),       ! high-pressure rate constant w/o tunneling effects
     &                   kcol,kosc)
             end if
 850       CONTINUE
         END IF
 900  CONTINUE

c------------------------------------------------------------------------------------------
c   WRITE OUT COLLISIONAL NORMALIZATION, UP-TRANSITION PROBABILITIES
c------------------------------------------------------------------------------------------

      DO 600 Mol = 1 , NWells
         Rob_1 = -1.0
         XRANST = PDOWN(Rob_1,Rob_1,Mol,Temp,SCALE)
         xref = Eref(Mol)
         IF ( ETKEY.Eq.oldet ) xref = 0.0
         DO ki = 7 , 9
           WRITE (ki,99057) Mol , ADJUSTR(MolName(Mol)),
     &      xref, (DC(Mol,i),i=1,8)
           WRITE (ki,99027) (LI(Mol,i),i=1,5)
         END DO
 600  CONTINUE
 
      WRITE (KARY,99030)
      WRITE (KARY,99012) (Mol,Mol,Mol=1,NWells)
      DO 700 II = 1 , Isize
         IF ( II.LE.imax1 ) THEN
            E = (II-1)*Egrain1
         ELSE
            E = (II-imax1-1)*Egrain2
         ENDIF
         WRITE (KARY,99011) E , 
     &        (CNORM(Mol,II),COLLUP(Mol,II),Mol=1,NWells)
 700  CONTINUE
 
c------------------------------------------------------------------------------------------
c   WRITE OUT RATE CONSTANT INFO TABLE
c------------------------------------------------------------------------------------------
      IF ( Nreac .GT. 0 ) THEN
      DO ki = 7 , 11                               ! Write to Units 7, 8, 9, 10, and 11
       WRITE (ki,99013)  Nreac
       WRITE (ki,99028) ((j,Jto(j,i),i=1,Nchan(j)),j=1,NWells)
       WRITE (ki,99034) ((ADJUSTR(TSname(j,i)),i=1,Nchan(j)),
     &   j=1,NWells)
       WRITE (ki,99045) ((TSmom(j,i),i=1,Nchan(j)),j=1,NWells)
       WRITE (ki,99046) ((TSsym(j,i),i=1,Nchan(j)),j=1,NWells)
       WRITE (ki,99047) ((TSele(j,i),i=1,Nchan(j)),j=1,NWells)
       WRITE (ki,99048) ((TSopt(j,i),i=1,Nchan(j)),j=1,NWells)
       WRITE (ki,99026) ((Path(j,i),i=1,Nchan(j)),j=1,NWells)
       WRITE (ki,99023) ((Hts(j,Jto(j,i)),i=1,Nchan(j)),j=1,NWells)
       WRITE (ki,99025) ((Hts(j,Jto(j,i))/jtocm,i=1,Nchan(j)),j=1,
     &                   NWells)
       WRITE (ki,99024) ((Hts(j,Jto(j,i))/caltocm,i=1,Nchan(j)),j=1,
     &                   NWells)
       WRITE (ki,99021) ((Eo(j,i),i=1,Nchan(j)),j=1,NWells)
       WRITE (ki,99022) ((Eo(j,i)/jtocm,i=1,Nchan(j)),j=1,NWells)
       WRITE (ki,99020) ((Eo(j,i)/caltocm,i=1,Nchan(j)),j=1,NWells)
       WRITE (ki,99029) ((Kinf(j,i),i=1,Nchan(j)),j=1,NWells)
       WRITE (ki,99049) ((Ainf(j,i),i=1,Nchan(j)),j=1,NWells)
       WRITE (ki,99050) ((Einf(j,i),i=1,Nchan(j)),j=1,NWells)
       WRITE (ki,99051) ((Einf(j,i)*caltoj,i=1,Nchan(j)),j=1,NWells)
       WRITE (ki,99086) ((kosc(j,i),i=1,Nchan(j)),j=1,NWells)
       WRITE (ki,99115) ((NCENT(j,i),i=1,Nchan(j)),j=1,NWells)
       WRITE (ki,99114) ((Eor(j,i),i=1,Nchan(j)),j=1,NWells)
       WRITE (ki,99052) ((Jread(j,i),i=1,Nchan(j)),j=1,NWells)
       WRITE (ki,99088) ((vimag(j,i),i=1,Nchan(j)),j=1,NWells)
       WRITE (ki,99090) ((WARN1(j,i),i=1,Nchan(j)),j=1,NWells)
       IF ( nivrflag .EQ. 1 ) THEN
c        WRITE (ki,99080) ((iivr(j,i),i=1,Nchan(j)),j=1,NWells)
         WRITE (ki,99081) ((vivr(j,i),i=1,Nchan(j)),j=1,NWells)
         WRITE (ki,99087) ((vave(j,i),i=1,Nchan(j)),j=1,NWells)
         WRITE (ki,99082) ((pivr(j,i),i=1,Nchan(j)),j=1,NWells)
         WRITE (ki,99186) ((tivr(j,i),i=1,Nchan(j)),j=1,NWells)
         WRITE (ki,99083) ((civr(j,i,1),i=1,Nchan(j)),j=1,NWells)
         WRITE (ki,99084) ((civr(j,i,2),i=1,Nchan(j)),j=1,NWells)
         WRITE (ki,99085) ((civr(j,i,3),i=1,Nchan(j)),j=1,NWells)
       ENDIF
       IF ( Pressflag .GT. 0 ) THEN
         WRITE (ki,99089) ((Press(j,i),i=1,Nchan(j)),j=1,NWells)
       ENDIF
       WRITE (ki,99053)

99153  FORMAT(//,
     & 'SUPPLEMENTARY Rate Constants (unimol, or bimol w. Bath Gas)',/,
     &  'From    To    TSname  rxn_order  Afac(cm^3 s-1)  Ea/R  ',
     &  'ksup(T:', F5.0,  ')  QuenchE(cm-1)')
99154  FORMAT(I3, 3x, I3, 3x, A10, 2x, I2, 7x, 1PE10.3 ,2x, 0PF8.2, 1x, 
     &        1PE10.3, 2x, 0pF8.0 )
       IF ( nsux .GT. 0 ) THEN							! if there are supplementary reactions
         WRITE (ki, 99153) Temp
         DO Mol = 1, NWells
           IF ( nsmax(Mol) .GT. 0 ) THEN
            DO i = 1, nsmax(Mol)
            WRITE (ki, 99154) Mol, sto( Mol,i ), SupName(Mol,i) ,  
     &       norder(Mol ,i), Afc(Mol, i), Bsr(Mol, i) , ksup( Mol,i ) , 
     &       dde( Mol,i )
            END DO   ! i
           END IF
         END DO   ! Mol
       END IF

      END DO  ! ki,  Write to Units 7, 8, 9, 10, and 11

      WRITE (KARY,99013)  Nreac
      WRITE (KARY,99113) ((Eor(j,i),i=1,Nchan(j)),j=1,NWells)
      WRITE (KARY,99016) ((j,Jto(j,i),i=1,Nchan(j)),j=1,NWells)

c------------------------------------------------------------------------------------------
c   WRITE OUT RATE CONSTANTS IN MULTIWELL.ARRAY OUTPUT FILE
c------------------------------------------------------------------------------------------
!     combine tunneling and nontunneling k(E) into KRate for output purposes

        do i=1,NWells
          do k=1,Nchan(i)
            do II=1,Isize+imax1
              if(II.le.imax1*2)then
                E=(II-1)*Egrain1-(imax1*Egrain1)
              else
                E=(II-imax1*2-1)*Egrain2
              end if
              if(itun(i,k).eq.1)then
                KRate(i,k,II)=TRate(i,k,II)
              else
                if(E.lt.0.0d0)then
                  KRate(i,k,II)=-5000.0d0     !ensures log[KRate(i,k,II)] is 0 below the rxn barrier
                else
                  III=II-imax1                !III index ensures Rate array for the nontunneling case
                  KRate(i,k,II)=Rate(i,k,III) !is stored correctly in the KRate array
                end if
              end if
            end do
          end do
        end do

!     output all k(E)

      IF ( ntunflag .EQ. 1 ) THEN                ! IF TUNNELING, write k(E) from below critical energy Eo

        do II=1,Isize+imax1
          if(II.le.imax1*2)then
            E=(II-1)*Egrain1-(imax1*Egrain1)
          else
            E=(II-imax1*2-1)*Egrain2
          end if
          WRITE(KARY,99018)E,
     &         ((EXP(KRate(j,i,II)),i=1,Nchan(j)),j=1,NWells)
        end do

      ELSE                                       ! IF NO TUNNELING, write k(E) starting at critical energy Eo

        do II= imax1+1 , Isize+imax1
          if(II.le.imax1*2)then
            E=(II-1)*Egrain1-(imax1*Egrain1)
          else
            E=(II-imax1*2-1)*Egrain2
          end if
          WRITE(KARY,99018)E,
     &         ((EXP(KRate(j,i,II)),i=1,Nchan(j)),j=1,NWells)
        end do

      ENDIF
      
      ENDIF     ! End printing when Nreac .GT. 0

 


c-------------------------------------------------------------------------------------
c-------------------------------------------------------------------------------------
c     Ntrials = number of trials
c     Tspec   = time specification (CHARACTER*4)
c             = 'TIME': Tread = Tlim (max time)
c             = 'COLL': Tread = max collisions, from which Tlim is calculated 
c     Tread   = maximum simulated time
c     KEYTYPE = KEYWORD  for type of initial distribution
c             = DELTA    for Monoenergetic at energy Einit          (Icalc=1)
c             = THERMAL  for Thermal with energy offset Einit       (Icalc=2)
c             = CHEMACT  for Chemical activation from 'product' #IR (Icalc=3)
c             = EXTERNAL for read from external file                (Icalc=4)
c     Molinit = index of initial molecule
c     IR      = origin of initial molecule, if produced via chemical activation
c     Einit   = initial energy (relative to ZPE of Molinit)
c-------------------------------------------------------------------------------------
c
      READ (KIN,*) Rtrials , Tspec , Tread , KEYTEMP , Molinit , IR , 
     &                Einit
      call ucase( Tspec )
      call ucase( KEYTEMP )
      Ntrials = NINT( Rtrials )
      IF ( KEYTEMP.EQ.'DELTA'   ) THEN
         Icalc = 1
        ELSEIF ( KEYTEMP.EQ.'THERMAL' ) THEN
         Icalc = 2
        ELSEIF ( KEYTEMP.EQ.'CHEMACT' ) THEN
         Icalc = 3
        ELSEIF ( KEYTEMP.EQ.'EXTERNAL' ) THEN
         Icalc = 4
        ELSE
         write(6,*)'FATAL: INITIAL ENERGY DISTRIBUTION NOT RECOGNIZED'
         write(KOUT,*)'FATAL: INITIAL ENRGY DISTRIBUTION NOT RECOGNIZED'
         STOP
      ENDIF

      IF ( Eunits.EQ.kcal ) THEN
         Einit = Einit*caltocm          ! Convert kcal/mole to cm-1
        ELSEIF ( Eunits.EQ.kjou ) THEN
         Einit = Einit*jtocm          ! Convert kJ/mole to cm-1
      ENDIF
  
         WRITE (KSMM,99059) Rtrials
         IF ( Icalc.EQ.1 ) THEN
          WRITE (KSMM,99165) Temp , ADJUSTR(MolName(Molinit)) , 
     &                         Einit , IDUM0 , IDUM
         ELSEIF ( Icalc.EQ.2 ) THEN
          WRITE (KSMM,99166) Tvib , Temp , ADJUSTR(MolName(Molinit)) , 
     &                         Einit , IDUM0 , IDUM
         ELSEIF ( Icalc.EQ.3 ) THEN
          WRITE (KSMM,99167) Tvib , Temp , ADJUSTR(MolName(Molinit)) ,
     &                         MolName(IR) , IDUM0 , IDUM
         ELSEIF ( Icalc.EQ.4 ) THEN
          WRITE (KSMM,99168) Temp , ADJUSTR(MolName(Molinit)) ,
     &                         IDUM0 , IDUM
         ENDIF
 
         WRITE (KSMM,99063) (ADJUSTR(MolName(i)),i=1,NWells+NProds)
         WRITE (KSMM,99060) (HMol(i),i=1,NWells)
         WRITE (KSMM,99161) (i,i,i=1,NWells+NProds)

c @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
c     Initialize Estart to calculate cumulative distribution
c
      E = Estart(0,Icalc,IR,Molinit,Einit,Tvib)

      Iwell=Molinit    ! save index for the well that the reaction will be initiated
                       ! from so that the correct Eor(Iwell,1) can be printed before the
                       ! cumulative distribution function info in multiwell.array
 
c --------------------------------------------------------------------
c
c     START PRESSURE STEPS & STOCHASTIC TRIALS
c
c	Igo	= flag which signals the following:
c		= 0	stop stepping for this trial
c		= 1	continue stepping for this trial
c
      DO 1200 Ip = 1 , Np               ! Pressure steps
         numbdens = P(Ip)*7.2428D+21/Temp
         CollFreq = CollK(Molinit)*numbdens

         IF ( Tspec.EQ.'TIME' ) THEN           ! Specify max time
            Tlim = Tread
            Tstep = Tlim/(Ntime-1)
         ELSEIF ( Tspec.EQ.'LOGT' ) THEN       ! Specify max time
            Tlim = Tread
            Tstep = log( Tlim / tminlogt )/(Ntime-1)  ! stepsize on log(t) scale from log(tminlogt) up to log(Tlim)
         ELSEIF ( Tspec.EQ.'COLL' ) THEN       ! Specify max number of collisions
           IF (CollFreq .GT. 0.0d+00) THEN
              Tlim = Tread/CollFreq
              Tstep = Tlim/(Ntime-1)
           ELSE
              Tlim = 100.
           ENDIF
         ELSE
           write(*,*) '** Keyword Tspec is not recognized ***'
           STOP (' *** FATAL: Tspec = TIME, COLL, or LOGT, only *** ')
         ENDIF

         IDUM0 = IDUM
 
         CALL INITIAL                   ! Initialize bookkeeping arrays
 
         M = 0
         WRITE (*,99164) PP(Ip)
         WRITE (*,99064) M , M
 
         DO 1050 M = 1 , Ntrials
            E = Estart(1,Icalc,IR,Molinit,Einit,Tvib)
            Igo = 1
            Mol = Molinit
            Time = 0.0D+00               ! Initialize
            DO WHILE ( Igo.EQ.1 )
               CALL STEPPER(Mol,E,Temp,numbdens,IPP,Delt)
               CALL BOOK1(Mol,E,Temp,IPP,Time,Delt,Igo)
               IF ( Time.GT.Tlim ) Igo = 0
            ENDDO
            Test = 20.*M/Ntrials - (20*M/Ntrials)
            IF ( ABS(Test).LT.1.E-06 ) WRITE (*,99064) M , 
     &           5*(20*M/Ntrials)
 1050    CONTINUE
c
c @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
c
c     Massage booked results and WRITE output for each pressure
c
c     KEYTYPE = KEYWORD  for type of initial distribution
c             = DELTA    for Monoenergetic at energy Einit          (Icalc=1)
c             = THERMAL  for Thermal with energy offset Einit       (Icalc=2)
c             = CHEMACT  for Chemical activation from 'product' #IR (Icalc=3)
c             = EXTERNAL for read from external file                (Icalc=4)
c
         CALL DateTime(KOUT)
         CALL DateTime(KARY)
 
         DO ki = 8 , 11
          WRITE (ki,99059) Rtrials
          IF ( Icalc.EQ.1 ) THEN                                        ! DELTA
            WRITE (ki,99065) Temp , PP(Ip) , POUT , numbdens , 
     &                    CollFreq , Eref(Molinit), 
     &                    ADJUSTR(MolName(Molinit)) , 
     &                    Einit , IDUM0 , IDUM
            ELSEIF ( Icalc.EQ.2 ) THEN                                  ! THERMAL
            WRITE (ki,99066) Tvib , Temp , PP(Ip) , POUT , numbdens , 
     &                    CollFreq , Eref(Molinit), 
     &                    ADJUSTR(MolName(Molinit)) , 
     &                         Einit , IDUM0 , IDUM
            ELSEIF ( Icalc.EQ.3 ) THEN                                  ! CHEMACT
            WRITE (ki,99067) Tvib , Temp , PP(Ip) , POUT , numbdens , 
     &                    CollFreq , Eref(Molinit), 
     &                    ADJUSTR(MolName(Molinit)) , 
     &                    ADJUSTR(MolName(IR)) , IDUM0 , IDUM
            ELSEIF ( Icalc.EQ.4 ) THEN                                  ! EXTERNAL
            WRITE (ki,99868) Temp , PP(Ip) , POUT , numbdens , 
     &                    CollFreq , Eref(Molinit), 
     &                    ADJUSTR(MolName(Molinit)) ,
     &                    IDUM0 , IDUM
         ENDIF
        END DO
c
c     Calculate populations and averages of each molecule
c
         WRITE (KOUT,99063) (ADJUSTR(MolName(i)),i=1,NWells+NProds)
         WRITE (KOUT,99060) (HMol(i),i=1,NWells)
         WRITE (KOUT,99061) (i,i,i=1,NWells+NProds)
         WRITE (KFLX,99017) ((j,Jto(j,i),i=1,Nchan(j)),j=1,NWells)
         WRITE (KRAT,99017) ((j,Jto(j,i),i=1,Nchan(j)),j=1,NWells)

         DO 1100 j = 1 , Ntime                         ! ************ Start of time-loop with Ntime time-steps **********
            IF ( Tspec.EQ.'TIME' .OR. Tspec.EQ.'COLL' ) THEN
                Time = (j-1)*Tstep
              ELSEIF ( Tspec.EQ.'LOGT' ) THEN
                Time = tminlogt*exp( (j-1)*Tstep )
            ENDIF
            
            NumTot = 0.0d+00
            DO 1060 i = 1 , NWells + NProds
               FracMol(i) = 0.0D+00
               AveE(i) = 0.0D+00
               NumTot = NumTot + REAL(Num(i,j),KIND=8)
 1060       CONTINUE
          IF ( NumTot .LT. 0.01 ) NumTot = 1.d+00
 
            DO 1080 Mol = 1 , NWells + NProds
               FracMol(Mol) = REAL(Num(Mol,j),KIND=8)/NumTot             ! Average population in each well
               Var(Mol) = SQRT(FracMol(Mol)*(1.0-FracMol(Mol))/NumTot)
               IF ( Mol.LE.NWells .AND. Num(Mol,j).GT.0 ) THEN
                 AveE(Mol) = EMol(Mol,j)/REAL( Num(Mol,j), KIND=8)               ! Average E in each well
                 DO 1065 k = 1 , Nchan(Mol)
                   kuni(Mol,k,j) = kuni(Mol,k,j)/REAL(Num(Mol,j),KIND=8)       ! average flux coefficients for a given path
                   Flux(Mol,k,j) = kuni(Mol,k,j)*FracMol(Mol)     ! reaction flux via a given path
 1065            CONTINUE
               ENDIF
 1080       CONTINUE
 
            WRITE (KOUT,99062) Time , Time*CollFreq , 
     &                         (FracMol(Mol),Var(Mol),AveE(Mol),Mol=1,
     &                         NWells+NProds)
            WRITE (KRAT,99019) Time , Time*CollFreq ,
     &                         ((kuni(Mol,k,j),k=1,Nchan(Mol)),Mol=1,
     &                         NWells)  ! average rate constants
            WRITE (KFLX,99019) Time , Time*CollFreq ,
     &                         ((Flux(Mol,k,j),k=1,Nchan(Mol)),Mol=1,
     &                         NWells)  ! average rate constants
 1100    CONTINUE ! End of time-step loop
 
         WRITE (KSMM,99062) numbdens , Tlim*CollFreq , 
     &                      (FracMol(Mol),Var(Mol),AveE(Mol),Mol=1,
     &                      NWells+NProds)
      

c ---------------------------------------------------------------------
c	Open and write out energy distributions
c
         OPEN (KDIS,FILE=NameDIST,STATUS='UNKNOWN')     ! vib distributions vs. time
         WRITE (KDIS,99001) AVERSION , ADATE , AVERSION , ADATE
         CALL DateTime(KDIS)
         WRITE (KDIS,99003) TITLE
         WRITE (KDIS,99044) Egrain1 , Emax1 , imax1 , Egrain2 , Emax2 , 
     &                   (Isize-imax1)

c    Write out energy distributions
         WRITE (KDIS,99070) Temp , Tvib , P(Ip) , (Mol, Mol=1,NWells)    ! Write out energy distributions
         Tstep = Tlim/(Mtime-1)
          DO j = 1 , Mtime
           kkmax = 0                           !------------- Find highest energy bin (kkmax) with non-zero population
           k = Ndist
           DO Mol = 1 , NWells
             IF (Mum(Mol,j) .EQ. 0 ) Mum(Mol,j) = 1			! to avoid divide-by-zero
             DO WHILE ( k.GT.1 .AND. EDIST(Mol,k,j).LE.0.0D+00 )
               k = k - 1
             ENDDO  ! k
             kkmax = MAX( kkmax, k )     ! highest energy bin (kkmax) with non-zero population
           END DO ! Mol                        !------------- 

           DO k = 1 , kkmax
             WRITE(KDIS,99071) (j-1)*Tstep, (j-1)*Tstep*Collfreq, 
     &         (k-1)*DistStep , 
     &         (EDIST(Mol,k,j)/REAL(Mum(Mol,j),KIND=8) ,  Mol=1,NWells)       ! Energy of Edist bin is at the TOP of the bin; first bin top = 0.0
           END DO  ! k

          END DO  ! j

         CLOSE (KDIS)                          ! vib distributions vs. time
c ---------------------------------------------------------------------

! this do loop ensures that the correct Eor and itun information are used when
! printing out the distributions below

        do i=1,Nchan(Molinit)
          IF ( Jto(Molinit,i).EQ.IR ) ichan = i
        end do

         IF ( Icalc.GT.1 ) THEN                              ! If not a delta function
            WRITE (KARY,99068)                               ! chemical activation or thermal distribution
            NCUM = 0
            II = 0
            DO WHILE ( (II .LE. Jsize) .AND. (NCUM .LT. Ntrials) )
              II = II + 1
              if (Icalc.eq.2 .OR. Icalc.eq.4) then
                 E = Einit + (II-1)*Hstart                   ! start at Einit  for THERMAL and EXTERNAL
              else if (Icalc.eq.3)then
                if(itun(Molinit,ichan).eq.0)then
                  E = Eor(Molinit,ichan) + (II-1)*Hstart     ! start at reaction threshold for CHEMACT w/o tunneling 
                else
                  Einit = Eo(Molinit,ichan) - Emax1
                  IF ( Einit .LT. 0.0 ) Einit = 0.0
                  E = Einit + (II-1)*Hstart                  ! start at Egrain1*imax1 below the reaction threshold for 
                end if                                       ! for CHEMACT with tunneling
              else
                 continue
              end if
              NCUM = Nstart(II) + NCUM                                ! cumulative
              WRITE (KARY,99069) E , Pstart(II) , 
     &            DBLE(NCUM)/DBLE(Ntrials)
            END DO

         ENDIF
 1200 CONTINUE                          ! End pressure step loop
c
c     FORMAT STATEMENTS
c
99001 FORMAT (
     &'%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
     &%%%%%%%%%%%%%'//8x,
     &'                                John R. Barker'/8x,
     &'      MultiWell-',A7,'         University of Michigan'/8x,
     &'                                Ann Arbor, MI 48109-2143'/8x,
     &'        ',A8,'                jrbarker@umich.edu'/8x,
     &'                                (734) 763 6239'//8x,
     &'      http://clasp-research.engin.umich.edu/multiwell/'//,
     &'%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
     &%%%%%%%%%%%%%'//8x,
     &'Suggested Literature Citations:'//4x,
     &'a) J.R. Barker, T.L. Nguyen, J.F. Stanton, C. Aieta, M. Ceotto,',
     &/7x,'F. Gabas, T.J.D. Kumar, C.G.L.Li, L.L.Lohr, A. Maranzana,',
     &/7x,'N.F. Ortiz, J.M. Preses, J.M. Simmie, J.A. Sonk, and ',
     &'P.J. Stimac; ',
     &/7x,'MultiWell-',A7,' Software Suite; University of Michigan, ',
     &/7x,'Ann Arbor, Michigan, USA, ',A8,'; ',
     &/7x,'http://clasp-research.engin.umich.edu/multiwell/.',
     &//4x,'b) J.R. Barker, Int. J. Chem. Kinetics, 33, 232-45 (2001)',
     &//4x,'c) J.R. Barker, Int. J. Chem. Kinetics, 41, 748-763 (2009)',
     &//,
     &'%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
     &%%%%%%%%%%%%%'/)
99002 FORMAT (/'Isize greater than maximum allowed: ',I5,
     &        ' elements.'/'****** Hit RETURN to terminate run *******')
99003 FORMAT (A100)
99004 FORMAT (60x,'DATE: ',I2.2,'/',I2.2,'/',I4.4,/,60x,'TIME: ',I2.2,
     &        ':',I2.2,':',I2.2//)
99005 FORMAT (//I2,1x,'Stable Molecules',//,
     &'No. Name             delH   2D-moment Symm Vibs Rots     ZPE     
     &  beta    <freq>  Thermal<E>')
99006 FORMAT (I2,2x,A10,1x,f10.1,2x,f10.1,2x,I2,2x,I2,2x,I2,1x,0pf10.1,
     &        1x,f10.4,1x,f10.3,1x,f10.1)
99007 FORMAT (I2,2x,A10,1x,f10.1,2x,f10.1,2x,I2,2x,
     &        'Densities from an external file',9x,f10.1)
99008 FORMAT (I2,2x,A10,1x,'      Irreversible product species')
99009 FORMAT (/'Units: 2D-moment : amu Ang^2',
     &         '       Thermal<E>: cm-1 (at Translational T)')
99010 FORMAT (/,5X,'Densities of States',/'     Energy',1x,150(1x,A10))
99011 FORMAT (1x,f10.1,150(1x,1pe10.3,1x,1pe10.3))
99012 FORMAT (/'     Energy',150(:'   Norm(',I2.2,') UpProb(',I2.2,')'))
99013 FORMAT (/,5X,I3,' REACTION RATE CONSTANTS: Canonical k(infinity)')
99113 FORMAT (7X,'(with centrifugal corrections) vs. active energy',/,/,
     &           '  Eocent:',150(1x,f10.1),/)
99114 FORMAT ('Eocent/cm-1:',150(1x,f10.1))
99115 FORMAT ('Centr.Corr.:',150(1x,I10))
99014 FORMAT (/,23X,I3,' Average Flux Coefficients',/22X,150(1X,A10))
99015 FORMAT (/,23X,I3,' Reactive Fluxes',/22X,150(1X,A10))
99016 FORMAT (' E-Eocent ',150(4X,I2.2,'...',I2.2))
99017 FORMAT ('  Time   Colls(Mol=1)',150(4X,I2.2,'...',I2.2))
99018 FORMAT (1x,f10.1,150(1x,1pe10.3))
99019 FORMAT (2(1x,1pe10.3),150(1x,1pe10.3))
99020 FORMAT ('   Eo(kcal):',150(1x,f10.2))
99121 FORMAT (12X,75(1X,A10))
99021 FORMAT ('   Eo(cm-1):',150(1x,f10.1))
99022 FORMAT ('     Eo(kJ):',150(1x,f10.2))
99023 FORMAT (' delH(cm-1):',150(1x,f10.1))
99024 FORMAT (' delH(kcal):',150(1x,f10.1))
99025 FORMAT ('   delH(kJ):',150(1x,f10.1))
99026 FORMAT ('Rx_Path*Qel:',150(1x,f10.1))
99027 FORMAT (A100)
99028 FORMAT ('       ',4x,150(4X,I2.2,'-->',I2.2))
99029 FORMAT ('     k(inf):',150(1x,1pe10.2))
99030 FORMAT (/)
99031 FORMAT (/,'TEMPERATURES:',/,
     &          '        Translational Temperature =',
     &        f8.1,1x,'K'/,    
     &          '  Initial Vibrational Temperature =',
     &        f8.1,1x,'K'///,I3,' WELLS and',I3,' PRODUCTS'/)
99032 FORMAT (/,I5,' PRODUCT SETS',/)
99033 FORMAT (' lowest_vib:',150(1x,f10.1))
99034 FORMAT ('    TS_Name:',150(1x,A10))
99035 FORMAT (' Molec_Name:',150(1x,A10))
99036 FORMAT ('  Molec_No.:',150(1x,I10))
99041 FORMAT (' Thermal<E>:',150(1x,f10.1))
99044 FORMAT (/'PARAMETERS FOR DOUBLE-ARRAYS:'/
     &    '        Grain (cm-1)      Maximum Energy    Number of grains'
     &    /2(2F20.3,i20/))
99045 FORMAT ('  2D-moment:',150(1x,f10.1))
99046 FORMAT ('   ext_symm:',150(1x,I10))
99047 FORMAT ('Qel_partfxn:',150(1x,F10.2))
99048 FORMAT ('   opt_isom:',150(1x,I10))
99049 FORMAT ('     A(inf):',150(1x,1pe10.1))
99050 FORMAT (' E(inf)kcal:',150(1x,f10.1))
99051 FORMAT ('   E(inf)kJ:',150(1x,f10.1))
99052 FORMAT ('     Source:',150(1x,I10))
99053 FORMAT ('-----------',/,
     &'SOURCE: 0=> ILT method; 1=> sums of states file; 2=> k(E) file;',
     &' -2=> for reverse rxn; 3=> CRP file.',/,
     &'CENTR.CORR.: 0=> none; 1=> 1-D adiabatic rotor;',
     &' 2=> 2-D adiabatic rotor; -2=> legacy (not recommended).',/,
     &'WARNING: number of energy grains where Reactant Well has ',
     &'states, but Product Well has none to match [set k(E) = 0].')
99054 FORMAT ('Temperature (K) =',f10.2,/,'Pressure (bar) =',1pe10.3/)
99055 FORMAT (/,'Lennard-Jones Parameters & Pdown Models    ',
     &        '[rate constants k(LJ) and k(QM) units: cc/sec]',//,
     &'Mol   Sig(Ang)      Eps/k MolWt(amu)     NetSig   NetEps/k    Red
     &Mass     k(LJ)    k(QM)     Used')
99056 FORMAT (1x,I2,6(1X,f10.3),3(1x,1pe10.3))
99057 FORMAT (/,'Well number: ',I2,5x,'Name: ',A10,/,
     &        'Reference energy for collision frequency:',0pf8.1,' cm-1',
     &        /,'Collision parameters:',/,8(1X,1pe10.3))
99058 FORMAT ('Inelastic collision parameters:',3(1X,1pe10.3))
99059 FORMAT (/'RESULTS FOR',1pe7.0,' TRIALS')
99060 FORMAT ('        deltaH(cm-1): ',75(5x,0pf10.1,18x))
99061 FORMAT ('   Time  Collisns(Mol=1)',
     &        75(: 3x,'Fract(',I2.2,') +/-Error  <Evib(',I2.2,')>'))
99161 FORMAT (3X ,'1/cc  Collisns(Mol=1)',
     &        75(: 3x,'Fract(',I2.2,') +/-Error  <Evib(',I2.2,')>'))
99062 FORMAT (2(1x,1pe10.3),75(3x, 1pe11.3 , 1pe9.1 , 0pf8.0,2x))
99063 FORMAT ('                Name:    ',75(5x,A10,18x))
99064 FORMAT (1x,I10,'  Trials = ',I5,' %')
99164 FORMAT (/,1x,'Pressure = ',1pe10.3)
99065 FORMAT ('****Monoenergetic****'/'    Ttrans  = ',f6.1,
     &        ' K',/,'    Press   = ',1pe10.3,1x,A4,/,'          N = ',
     &        1pe10.3,' /cm3',/,
     &        'Coll. Freq. = ',1pe10.3,' /s at E =',0pf8.1,' cm-1',/,
     &        'Initial Molecule: ',A10,/,
     &       'E(initial) =',
     &        0pf10.1,' cm-1',/,'Starting random number seed = ',
     &        I10,/,'Ending   random number seed = ',I10,/)
99066 FORMAT ('****Shifted Thermal (at Tvib)****'/'     Tvib   = ',f6.1,
     &        ' K',/,'     Ttrans = ',f6.1,' K',/,'     Press  = ',
     &        1pe10.3,
     &        1x,A4,/,'          N = ',1pe10.3,' /cm3',/,
     &        'Coll. Freq. = ',1pe10.3,' /s at E =',0pf8.1,' cm-1',/,
     &        'Initial Molecule: ',A10,/,'E(shift) =',
     &        0pf10.1,' cm-1'/'Starting random number seed = ',
     &        I10,/,'Ending   random number seed = ',I10,/)
99067 FORMAT ('****Chemical Activation (at Tvib)****'/'     Tvib   = ',
     &        f6.1,' K',/,'     Ttrans = ',f6.1,' K',/,'     Press  = ',
     &        1pe10.3,1x,A4,/'          N = ',
     &        1pe10.3,' /cm3',/,
     &        'Coll. Freq. = ',1pe10.3,' /s at E =',0pf8.1,' cm-1',/,
     &        'Excited Molecule: ',
     &        A10,/,'   Produced from: ',
     &        A10,/,'Starting random number seed = ',
     &        I10,/,'Ending   random number seed = ',I10,/)
99868 FORMAT ('**** ENERGY DISTRIBUTION FROM EXTERNAL FILE ****'/,
     &        '     Ttrans = ',f6.1,' K',/,'     Press  = ',
     &        1pe10.3,1x,A4,/'          N = ',
     &        1pe10.3,' /cm3',/,
     &        'Coll. Freq. = ',1pe10.3,' /s at E =',0pf10.1,' cm-1',/,
     &        'Initial Molecule: ',
     &        A10,/,'Starting random number seed = ',
     &        I10,/,'Ending   random number seed = ',I10,/)
99165 FORMAT (/,/,'****Monoenergetic****'/'    Ttrans  = ',f6.1,
     &        ' K',/,'Initial Molecule: ',A10,/,'E(initial) =',
     &        0pf10.1,' cm-1',/,'Starting random number seed = ',
     &        I10,/,'Ending   random number seed = ',I10,/)
99166 FORMAT (/,/,'****Shifted Thermal (at Tvib)****'/'     Tvib   = ',
     &        f6.1,
     &        ' K',/,'     Ttrans = ',f6.1,' K',/,'Initial Molecule: ',
     &        A10,/,'E(shift) =',
     &        0pf10.1,' cm-1',/,'Starting random number seed = ',
     &        I10,/,'Ending   random number seed = ',I10,/)
99167 FORMAT (/,/,'****Chemical Activation (at Tvib)****'
     &        /'     Tvib   = ',
     &        f6.1,' K',/,'     Ttrans = ',f6.1,' K',/,
     &        'Excited Molecule: ',
     &        A10,/,'   Produced from: ',
     &        A10,/,'Starting random number seed = ',
     &        I10,/,'Ending   random number seed = ',I10,/)
99168 FORMAT (/,/,'**** Energy Distribution from an EXTERNAL File ****',
     &        /,'     Ttrans = ',f6.1,' K',/,
     &        A10,/,'Starting random number seed = ',
     &        I10,/,'Ending   random number seed = ',I10,/)
99068 FORMAT (/'CUMULATIVE DISTRIBUTION FXNS FOR STARTING ENERGIES',
     &        ' (Pstart)',//,
     &        '    ENERGY   P(calc)    P(actual) ')
99069 FORMAT (1x,f10.1,2(1x,1pe10.3))
99070 FORMAT (/'Energy Distribution (non-zero elements)',/,
     & '  (E = Active Energy at TOP of energy bin, relative to ZPE)',
     & //,' Ttrans=',f6.1,1x,'K',5x,'Tvib=',f6.1,1x,'K',5x,
     & 'P= ',1PE10.3,1x,'bar',//,
     & ' Time       Collisions    E(cm-1)    ',75(: 'Well=',I2.2,4x))
99071 FORMAT (1x,1pe10.3,2(1x,0pf10.1),75(1x,1pe10.3) )
99072 FORMAT (8x,I3,2x,A10,2x,'delH =',f10.1,' cm-1')
99076 FORMAT (8x,I3,2x,A10,2x,'delH =',f10.1,' kJ/mol')
99077 FORMAT (8x,I3,2x,A10,2x,'delH =',f10.1,' kcal/mol')
99073 FORMAT (//,'COLLISION PARAMETERS FOR ',I2,' WELLS')
99074 FORMAT (/,'INPUT DATA FILE CHECK: ',I2,' REACTION CHANNELS'/
     &'#  Mol ito TS          RR       j       Qel   l   AA         ',
     &'EE       KEYWORDS')
99075 FORMAT(3(I2,1X),A10,1X,1PE10.2,1X,I2,1X,0PF10.2,1X,I2,
     &  2(1PE10.2,1X),5(A7,1x))
99080 FORMAT ('IVR: 0 or 1:',150(1x,I10))
99081 FORMAT ('IVR  v(IVR):',150(1x,f10.1),' cm-1')
99087 FORMAT ('IVR  v(ave):',150(1x,f10.1),' cm-1')
99082 FORMAT ('IVR  coll k:',150(1x,1pe10.3))
99186 FORMAT ('IVR  thresh:',150(1x,f10.1))
99083 FORMAT ('IVR civr(1):',150(1x,1pe10.3))
99084 FORMAT ('IVR civr(2):',150(1x,1pe10.3))
99085 FORMAT ('IVR civr(3):',150(1x,1pe10.3))
99086 FORMAT (' koSC(cc/s):',150(1x,1pe10.2))
99088 FORMAT ('TUN   vimag:',150(1x,0pf10.1))
99089 FORMAT ('React. [B] :',150(1x,1pe10.2))
99090 FORMAT (' WARNING!!!:',150(1x,I10))
99173 FORMAT ( A5 )
      END
