 
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c
c MultiWell: a code for chemical kinetics calculations.
c Copyright (C) 2002 John R. Barker
c
c John R. Barker
c jrbarker@umich.edu
c University of Michigan
c Ann Arbor, MI 48109-2143
c (734) 763 6239
c
c This program is free software; you can redistribute it and/or
c modify it under the terms of the GNU General Public License (version 2)
c as published by the Free Software Foundation.
c
c This program is distributed in the hope that it will be useful,
c but WITHOUT ANY WARRANTY; without even the implied warranty of
c MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
c GNU General Public License for more details.
c
c See the 'ReadMe' file for a copy of the GNU General Public License,
c or contact:
c
c Free Software Foundation, Inc.
c 59 Temple Place - Suite 330
c Boston, MA 02111-1307, USA.
c
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c
c     01/02  v1.2.1  -New Feature: can optionally choose to apply centrifugal
c                     corrections for any choice of model. Thus can apply them
c                     even when using Inverse Laplace Transform form of k(E).
c                     NOTE: this new feature affects the data file format.
c     11/01  v1.1.4  -Bug Fix: previous version applied centrifugal correction 
c                      inconsistently when calculating Arrhenius parameters 
c                     (k(inf) was correct, however).
c     09/01  v1.1.2  -Revised centrigfugal corrections to k(E)'s.
c                     See Robinson & Holbrook (1972) eq. 4.30 and 4.31.
c
c***********************************************************************
c	Carries out Gillespie Exact Stochastic procedure for one step
c
c	Mol	= molecule index
c	E	= energy (relative to ZPE of Mol)
c	T	= translational temperature
c	NumbDens= number density of collider gas
c	IP	= index of selected process
c	Delt	= time step
c***********************************************************************
      SUBROUTINE STEPPER(Mol,E,T,NumbDens,IP,Delt)
      IMPLICIT NONE
 
      INCLUDE 'declare1.inc'
      INCLUDE 'declare2.inc'
 
      INTEGER Mol , i , IE , IP , IPtot , J , SKIP
 
      DOUBLE PRECISION Atot , RX , arg , E , X , Delt , Atemp , RAN1 , 
     &                 Y1 , Y2 , Y3 , Y , T , numbdens , Aref , CC ,
     &                 DOUBNTERP, CTEST, Enew , rsup
 
      DOUBLE PRECISION vivr, vave, civr , rivr , pivr, tivr, arg2 , RVR

      INTEGER iivr

      COMMON/IVR/ vivr( MaxWell , MaxChan ) , 
     &            vave( MaxWell , MaxChan ) ,
     &            pivr( MaxWell , MaxChan ) ,
     &            tivr( MaxWell , MaxChan ) ,
     &            civr( MaxWell , MaxChan , 3 ) , 
     &            iivr( MaxWell , MaxChan )

      EXTERNAL DOUBNTERP

      SAVE 

      IF ( ETKEY.EQ.newet ) THEN                      ! Barker's NEW energy transfer treatment <<<<<<<<<<
        CC = DOUBNTERP(E,Mol,CNORM)/CNORMref(Mol)
      ELSEIF ( ETKEY.EQ.oldet ) THEN                  ! Traditional energy transfer treatment
        CC = 1.0d+00                                  
      ENDIF

      IPtot = Nchan(Mol) + nsmax(Mol) + 1		! rxn channels, supplemenary rxns, and collisions for a given Mol
      Rab(IPtot) = CollK(Mol)*numbdens*CC           ! Inelastic collision frequency: includes NEW up-transition probability
      Atot = Rab(IPtot)                             ! Collision frequency
      IF ( Nchan(Mol).GT.0 ) THEN                          ! If reaction channels
        DO i = 1 , Nchan(Mol)
          Rab(i) = 0.0D+00
          arg = E - Eor(Mol,i)                           ! Active energy in TS to determine k(E); centrifugal correction
          call NTERP2(arg,IE,X)
          arg2 =-imax1*Egrain1                           ! minimum active energy in TS for the tunneling case
          IF((itun(Mol,i).eq.1).and.(arg.ge.arg2))then   ! TUNNELING
            Y1 = log(rivr(Mol,i,IE-1,numbdens)) + TRate(Mol,i,IE-1)  ! log (IVR transmission coefficient * k(E))
            Y2 = log(rivr(Mol,i,IE,numbdens))   + TRate(Mol,i,IE)    ! log (IVR transmission coefficient * k(E))
            Y3 = log(rivr(Mol,i,IE+1,numbdens)) + TRate(Mol,i,IE+1)  ! log (IVR transmission coefficient * k(E))
            CALL QUDINT(Y1,Y2,Y3,X,Y) 
            Rab(i) = EXP(Y)           ! reminder: k(E) stored as log [now includes IVR effects]
          ELSEIF ( arg .GE. 0.0D+00 ) THEN                 ! NO TUNNELING and above rxn threshold energy
            Rab(i) = RVR( Mol , i , arg )                 ! k(E) obtained by interpolation
     &                 * rivr( Mol , i , IE , numbdens )  ! IVR transmission coefficient
          ENDIF
          Atot = Atot + Rab(i)
        END DO
         
      ENDIF
      
      IF ( nsmax(Mol) .GT. 0 ) THEN				! If canonical supplementary reactions
         DO i = 1, nsmax(Mol)
           IF ( norder(Mol,i) .EQ. 1) THEN			! order of reaction
              rsup = ksup( Mol, i )				! first-order 
            ELSEIF ( norder(Mol,i) .EQ. 2) THEN
              rsup = numbdens*ksup( Mol, i )			! PSEUDO-first-order (2nd order rate constant)
           ENDIF
           Rab( Nchan(Mol) + i ) = rsup
           Atot = Atot + rsup			   			! includes supplementary rxn channels
        END DO
      ENDIF   ! nsmax(Mol)

      IF ( Atot .GT. 1.e-15 ) THEN      ! Probably small enough for most applications.....
        RX = RAN1(IDUM)
        Aref = RX*Atot
        Atemp = 0.0D+00        
        IP = Nchan(Mol) + nsmax(Mol) + 2				      ! includes supplementary rxn channels
        DO WHILE ( (Atemp.LE.Aref) .AND. (IP .GT. 1 ) )	! select process; IP = process number (index)
          IP = IP - 1
          Atemp = Atemp + Rab(IP)
        ENDDO         
        RX = RAN1(IDUM)                           	! fresh random number
        IF ( RX .LE. 1.00D-10 ) RX = RAN1(IDUM)   	! if random number too small, call another one
        Delt = -LOG(RX)/Atot                      	! Determine time step
      ELSE
        IP =  Nchan(Mol) + nsmax(Mol) + 1			! i.e., collisions
        Delt = Tlim
      ENDIF

      RETURN
      END
